@@ -24,7 +24,7 @@
 
 extern const char *minor_str[];
 
-#define VERSION "v7.3"
+#define VERSION "v7.5"
 
 #ifndef O_DIRECT
 #define O_DIRECT	040000	/* direct disk access, not easily obtained from headers */
@@ -99,8 +99,9 @@ static int	set_acoustic = 0, get_acoustic = 0, acoustic = 0;
 
 static int	get_doreset = 0, set_doreset = 0;
 static int	get_tristate = 0, set_tristate = 0, tristate = 0;
+static int	i_know_what_i_am_doing = 0;
 
-static int open_flags = O_RDONLY|O_NONBLOCK;
+static int open_flags = O_RDWR|O_NONBLOCK;
 
 // Historically, if there was no HDIO_OBSOLETE_IDENTITY, then
 // then the HDIO_GET_IDENTITY only returned 142 bytes.
@@ -266,12 +267,17 @@ static void dump_identity (__u16 *idw)
 	printf("\n");
 }
 
+#ifndef ENOIOCTLCMD
+#define ENOIOCTLCMD ENOTTY
+#endif
+
 void flush_buffer_cache (int fd)
 {
 	fsync (fd);				/* flush buffers */
 	if (ioctl(fd, BLKFLSBUF, NULL))		/* do it again, big time */
 		perror("BLKFLSBUF failed");
-	if (do_drive_cmd(fd, NULL) && errno != EINVAL)	/* await completion */
+	/* await completion */
+	if (do_drive_cmd(fd, NULL) && errno != EINVAL && errno != ENOTTY && errno != ENOIOCTLCMD)
 		perror("HDIO_DRIVE_CMD(null) (wait for flush complete) failed");
 }
 
@@ -784,6 +790,16 @@ static void *get_identify_data (int fd, void *prev)
 	return id;
 }
 
+static void confirm_i_know_what_i_am_doing (const char *opt, const char *explanation)
+{
+	if (!i_know_what_i_am_doing) {
+		fprintf(stderr, "Use of %s is VERY DANGEROUS.\n%s\n"
+		"Please supply the --yes-i-know-what-i-am-doing flag if you really want this\n"
+		"Program aborted\n", opt, explanation);
+		exit(EPERM);
+	}
+}
+
 void process_dev (char *devname)
 {
 	int fd;
@@ -958,6 +974,9 @@ void process_dev (char *devname)
 			printf(" spin-up:");
 			fflush(stdout);
 			(void) do_drive_cmd(fd, args1);
+		} else {
+			confirm_i_know_what_i_am_doing("-s1",
+				"This requires BIOS and kernel support to recognize/boot the drive.");
 		}
 		if (get_powerup_in_standby) {
 			printf(" setting power-up in standby to %d", powerup_in_standby);
@@ -1555,27 +1574,22 @@ handle_standalone_longarg (char *name)
 	} else if (0 == strcasecmp(name, "security-freeze")) {
 		set_freeze = 1;
 	} else if (0 == strcasecmp(name, "security-unlock")) {
-		open_flags |= O_RDWR;
 		set_security = 1;
 		security_command = ATA_OP_SECURITY_UNLOCK;
 		get_security_password(0);
 	} else if (0 == strcasecmp(name, "security-set-pass")) {
-		open_flags |= O_RDWR;
 		set_security = 1;
 		security_command = ATA_OP_SECURITY_SET_PASS;
 		get_security_password(1);
 	} else if (0 == strcasecmp(name, "security-disable")) {
-		open_flags |= O_RDWR;
 		set_security = 1;
 		security_command = ATA_OP_SECURITY_DISABLE;
 		get_security_password(1);
 	} else if (0 == strcasecmp(name, "security-erase")) {
-		open_flags |= O_RDWR;
 		set_security = 1;
 		security_command = ATA_OP_SECURITY_ERASE_UNIT;
 		get_security_password(1);
 	} else if (0 == strcasecmp(name, "security-erase-enhanced")) {
-		open_flags |= O_RDWR;
 		set_security = 1;
 		enhanced_erase = 1;
 		security_command = ATA_OP_SECURITY_ERASE_UNIT;
@@ -1595,8 +1609,12 @@ get_longarg (void)
 	if (0 == strcasecmp(name, "verbose")) {
 		verbose = 1;
 		--num_flags_processed;	/* doesn't count as an action flag */
+	} else if (0 == strcasecmp(name, "yes-i-know-what-i-am-doing")) {
+		i_know_what_i_am_doing = 1;
+		--num_flags_processed;	/* doesn't count as an action flag */
 	} else if (0 == strcasecmp(name, "direct")) {
 		open_flags |= O_DIRECT;
+		--num_flags_processed;	/* doesn't count as an action flag */
 	} else if (0 == strcasecmp(name, "drq-hsm-error")) {
 		do_drq_hsm_error = 1;
 	} else if (0 == strcasecmp(name, "Istdout")) {
